
module.exports = vwsUtils();

function vwsUtils() {

	var myScope = this;

	this.store = {
		class2type: {}
	}

	this.regex = {
		rdigit: /\d/
	}

	this.extend = function() {
		var options, name, src, copy, copyIsArray, clone,
			target = arguments[0] || {},
			i = 1,
			length = arguments.length,
			deep = false;

		// Handle a deep copy situation
		if ( typeof target === "boolean" ) {
			deep = target;
			target = arguments[1] || {};
			// skip the boolean and the target
			i = 2;
		}

		// Handle case when target is a string or something (possible in deep copy)
		if ( typeof target !== "object" && !myScope.isFunction(target) ) {
			target = {};
		}

		// extend jQuery itself if only one argument is passed
		if ( length === i ) {
			target = myScope;
			--i;
		}

		for ( ; i < length; i++ ) {
			// Only deal with non-null/undefined values
			if ( (options = arguments[ i ]) != null ) {
				// Extend the base object
				for ( name in options ) {
					src = target[ name ];
					copy = options[ name ];

					// Prevent never-ending loop
					if ( target === copy ) {
						continue;
					}

					// Recurse if we're merging plain objects or arrays
					if ( deep && copy && ( myScope.isPlainObject(copy) || (copyIsArray = myScope.isArray(copy)) ) ) {
						if ( copyIsArray ) {
							copyIsArray = false;
							clone = src && myScope.isArray(src) ? src : [];

						} else {
							clone = src && myScope.isPlainObject(src) ? src : {};
						}

						// Never move original objects, clone them
						target[ name ] = myScope.extend( deep, clone, copy );

					// Don't bring in undefined values
					} else if ( copy !== undefined ) {
						target[ name ] = copy;
					}
				}
			}
		}
		// Return the modified object
		return target;
	}

	this.isFunction = function( obj ) {
		return myScope.type(obj) === "function";
	};

	this.isArray = Array.isArray || function( obj ) {
		return myScope.type(obj) === "array";
	};

	// A crude way of determining if an object is a window
	this.isWindow = function( obj ) {
		return obj && typeof obj === "object" && "setInterval" in obj;
	};

	this.isNaN = function( obj ) {
		return obj == null || !myScope.regex.rdigit.test( obj ) || isNaN( obj );
	};
	this.type = function( obj ) {
		return obj == null ?
			String( obj ) :
			this.store.class2type[ this.toString.call(obj) ] || "object";
	};

	this.isPlainObject = function( obj ) {
		// Must be an Object.
		// Because of IE, we also have to check the presence of the constructor property.
		// Make sure that DOM nodes and window objects don't pass through, as well
		if ( !obj || myScope.type(obj) !== "object" || obj.nodeType || myScope.isWindow( obj ) ) {
			return false;
		}

		// Not own constructor property must be Object
		if ( obj.constructor &&
			!myScope.hasOwn.call(obj, "constructor") &&
			!myScope.hasOwn.call(obj.constructor.prototype, "isPrototypeOf") ) {
			return false;
		}

		// Own properties are enumerated firstly, so to speed up,
		// if last one is own, then all properties are own.

		var key;
		for ( key in obj ) {}

		return key === undefined || myScope.hasOwn.call( obj, key );
	};

	this.isEmptyObject = function( obj ) {
		for ( var name in obj ) {
			return false;
		}
		return true;
	}

	hasOwn = Object.prototype.hasOwnProperty;
	toString = Object.prototype.toString;
	var classes = "Boolean Number String Function Array Date RegExp Object".split(" ");
	for (var i in classes) {
		this.store.class2type[ "[object " + classes[i] + "]" ] = classes[i].toLowerCase();
	}
	return this;
}
